@@ -16,19 +16,32 @@
 """This module contains classes and constants related to outputting test results."""
 __testify = 1
 
-import datetime
-import operator
-import traceback
-import sys
-import logging
 import collections
-from IPython import ultraTB
+import datetime
+import logging
+import operator
+import subprocess
+import sys
+import traceback
+
+# If IPython is available, use it for fancy color traceback formatting
+try:
+    try:
+        # IPython >= 0.11
+        from IPython.core.ultratb import ColorTB
+        _hush_pyflakes = [ColorTB]
+        del _hush_pyflakes
+    except ImportError:
+        # IPython < 0.11
+        from IPython.ultraTB import ColorTB
+
+    fancy_tb_formatter = staticmethod(ColorTB().text)
+except ImportError:
+    fancy_tb_formatter = staticmethod(traceback.format_exception)
 
 from testify import test_reporter
 from testify.test_case import TestCase
 
-# from test_case import TestCase
-
 # Beyond the nicely formatted test output provided by the test logger classes, we
 # also want to make basic test running /result info available via standard python logger
 _log = logging.getLogger('testify')
@@ -38,7 +51,7 @@ VERBOSITY_NORMAL    = 1  # Output dots for each test method run
 VERBOSITY_VERBOSE   = 2  # Output method names and timing information
 
 class TestLoggerBase(test_reporter.TestReporter):
-    traceback_formater = staticmethod(traceback.format_exception)
+    traceback_formatter = staticmethod(traceback.format_exception)
 
     def __init__(self, options, stream=sys.stdout):
         super(TestLoggerBase, self).__init__(options)
@@ -57,7 +70,7 @@ class TestLoggerBase(test_reporter.TestReporter):
                 self.report_test_result(result)
             self.results.append(result)
         elif result.test_method._fixture_type == 'class_teardown' and (result.failure or result.error):
-            # For a class_teardown failure, log the name too (since it wouldn't have 
+            # For a class_teardown failure, log the name too (since it wouldn't have
             # already been logged by on_run_test_method).
             self.report_test_name(result.test_method)
             self.report_test_result(result)
@@ -89,7 +102,7 @@ class TestLoggerBase(test_reporter.TestReporter):
 
         return bool((len(results_by_status['failed']) + len(results_by_status['unknown'])) == 0)
 
-    def report_test_name(self, test_name): 
+    def report_test_name(self, test_name):
         pass
     def report_test_result(self, result):
         pass
@@ -99,7 +112,7 @@ class TestLoggerBase(test_reporter.TestReporter):
             'FAILURES': [],
             'EXPECTED_FAILURES': []
             }
-        
+
         [results['EXPECTED_FAILURES'].append(result) if result.expected_failure else results['FAILURES'].append(result) for result in failed_results]
 
         if results['EXPECTED_FAILURES']:
@@ -116,8 +129,8 @@ class TestLoggerBase(test_reporter.TestReporter):
             # output won't have to scroll up to figure out whether failures
             # were expected or not.
             self.heading('FAILURES', 'None!')
-        
-    def report_failure(self, result): 
+
+    def report_failure(self, result):
         pass
 
     def report_stats(self, test_case_count, all_results, failed_results, unknown_results):
@@ -130,7 +143,7 @@ class TestLoggerBase(test_reporter.TestReporter):
             out.append("%s " % test_method.im_class.__module__)
         out.append("%s.%s" % (test_method.im_class.__name__, test_method.__name__))
 
-        return''.join(out)
+        return ''.join(out)
 
     # Helper methods for extracting relevant entries from a stack trace
     def _format_exception_info(self, exception_info_tuple):
@@ -141,12 +154,12 @@ class TestLoggerBase(test_reporter.TestReporter):
         if exctype is AssertionError:
             # Skip testify.assertions traceback levels
             length = self.__count_relevant_tb_levels(tb)
-            return self.traceback_formater(exctype, value, tb, length)
+            return self.traceback_formatter(exctype, value, tb, length)
 
         if not tb:
             return "Exception: %r (%r)" % (exctype, value)
 
-        return self.traceback_formater(exctype, value, tb)
+        return self.traceback_formatter(exctype, value, tb)
 
     def __is_relevant_tb_level(self, tb):
         return tb.tb_frame.f_globals.has_key('__testify')
@@ -160,7 +173,22 @@ class TestLoggerBase(test_reporter.TestReporter):
 
 
 class TextTestLogger(TestLoggerBase):
-    traceback_formater = staticmethod(ultraTB.ColorTB().text)
+    traceback_formatter = fancy_tb_formatter
+    def __init__(self, options, stream=sys.stdout):
+        super(TextTestLogger, self).__init__(options)
+
+        # Checking for color support isn't as fun as we might hope.  We're
+        # going to use the command 'tput colors' to get a list of colors
+        # supported by the shell. But of course we if this fails terribly,
+        # we'll want to just fall back to no colors
+        self.use_color = False
+        if sys.stdin.isatty():
+            try:
+                output = subprocess.Popen(["tput", "colors"], stdout=subprocess.PIPE).communicate()[0]
+                if int(output.strip()) >= 8:
+                    self.use_color = True
+            except Exception, e:
+                _log.debug("Failed to find color support: %r", e)
 
     def write(self, message):
         """Write a message to the output stream, no trailing newline"""
@@ -175,7 +203,7 @@ class TextTestLogger(TestLoggerBase):
     BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(30, 38)
 
     def _colorize(self, message, color = CYAN):
-        if not color:
+        if not color or not self.use_color:
             return message
         else:
             start_color = chr(0033) + '[1;%sm' % color
@@ -288,13 +316,13 @@ class TextTestLogger(TestLoggerBase):
         self.write("%s, %s.  " % (passed_string, failed_string))
 
         total_test_time = reduce(
-            operator.add, 
-            (result.run_time for result in (successful+unexpected_success+failed+incomplete)), 
+            operator.add,
+            (result.run_time for result in (successful+unexpected_success+failed+incomplete)),
             datetime.timedelta())
         self.writeln("(Total test time %.2fs)" % (total_test_time.seconds + total_test_time.microseconds / 1000000.0))
 
 class HTMLTestLogger(TextTestLogger):
-    traceback_formater = staticmethod(traceback.format_exception)
+    traceback_formatter = staticmethod(traceback.format_exception)
 
     def writeln(self, message):
         """Write a message and append a newline"""
@@ -319,7 +347,7 @@ class HTMLTestLogger(TextTestLogger):
             return start_color + message + end_color
 
 class ColorlessTextTestLogger(TextTestLogger):
-    traceback_formater = staticmethod(traceback.format_exception)
+    traceback_formatter = staticmethod(traceback.format_exception)
 
     def _colorize(self, message, color=None):
         return message
@@ -328,4 +356,4 @@ class ColorlessTextTestLogger(TextTestLogger):
 class TestResultGrabberHandler(logging.Handler):
     """Logging handler to store log message during a test run"""
     def emit(self, record):
-        raise Exception(repr(record))
\ No newline at end of file
+        raise Exception(repr(record))
